//=============================================================================
//=
//=     *** Kemper LED Lamp ***					10/29/2009
//=
//=============================================================================

/*
 Powerhouse Electronics 2010, Some Rights Reserved 
 Kemper LED Lamp 2010, Some Rights Reserved 
    Except where otherwise noted, this work is licensed under 
    This document is licensed under the Attribution-NonCommercial-ShareAlike 3.0 
    Unported license, available at http://creativecommons.org/licenses/by-nc-sa/3.0/.

You are free:
   * to Share  to copy, distribute and transmit the work
   * to Remix  to adapt the work
   * not for Commercial use unless presission is given.

Under the following conditions:
   * Attribution. You must attribute the work in the manner specified by the author 
   or licensor (but not in any way that suggests that they endorse you or your use 
   of the work).
   * Share Alike. If you alter, transform, or build upon this work, you may distribute 
   the resulting work only under the same, similar or a compatible license.
   * For any reuse or distribution, you must make clear to others the license terms 
   of this work. The best way to do this is with a link to this web page.
   * Any of the above conditions can be waived if you get permission from the copyright
   holder.
   * Nothing in this license impairs or restricts the author's moral rights.
*/

#include <12F609.h>

#pragma fuses IOSC8, MCLR, INTRC_IO, PROTECT

#use delay(clock=8M)
#use fast_io( ALL )

#include "node.h"

// ========================================================
// Note, all types (except floats) are unsiged by default!
// ========================================================
//typedef unsigned int8 uint8;
//typedef unsigned int16 uint16;
typedef unsigned int1 bool;

#BYTE	TMR0 	= 0x01		// Timer0 Register
#BYTE	STATUS 	= 0x03		// Status Register
#BIT 	MATHOF 	= 0x03.0	// Math Over/Under flow bit in status register.
#BYTE 	GPIO 	= 0x05		// General I/O Port
#BIT 	GP0  	= 0x05.0	// LED 0
#BIT 	GP1  	= 0x05.1	// LED 1
#BIT 	GP2  	= 0x05.2	// LED 2
#BIT 	GP3	= 0x05.3	    // MCLR / Unused
#BIT 	GP4  	= 0x05.4	// Comm Pin
#BIT 	GP5  	= 0x05.5	// LED 3 Aux
#BYTE	PCLATH 	= 0x0A		// PC Latch
#BYTE	INTCON 	= 0x0B		// INTERRUPT CONTROL REGISTER
#BIT	GPIF 	= 0x0B.0	// GPIO Change Interrupt Flag bit
#BIT	INTF	= 0x0B.1	// GP2/INT External Interrupt Flag bit
#BIT 	T0IF 	= 0x0B.2	// Timer0 Overflow Flag
#BIT	GIE 	= 0x0B.7	// Global Interrupt Enable bit
#BYTE 	PIR1 	= 0x0C		// PERIPHERAL INTERRUPT REQUEST REGISTER 1
#BYTE	TMR1L 	= 0x0E		// Timer1 Low Byte
#BYTE	TMR1H 	= 0x0F		// Timer1 High Byte
#BIT	GPPU	= 0x81.7	// Global Pullup Enable (active low)
#BYTE	TRISIO	= 0x85		// GPIO TRI-STATE REGISTER ( 1's=Input)
#BYTE	PIE1 	= 0x8C		// PERIPHERAL INTERRUPT ENABLE REGISTER 1
#BYTE	WPU	= 0x95		    // Weak Pullup Control register.
#BYTE	IOC 	= 0x96		// Interrupt-on-Change GPIO Register
#BYTE	ANSEL 	= 0x9F		// Analog Select Register

// More human readable / memorable names. 
#BIT 	pBLU  	= 0x05.0	// LED 0
#BIT 	pGRN  	= 0x05.1	// LED 1
#BIT 	pCOMM  	= 0x05.2	// Comm Pin
#BIT 	pRED  	= 0x05.4	// LED 2
#BIT 	pWHT  	= 0x05.5	// LED 3

// Data Direction Control Bit for Comm Pin.
#BIT	pDDR_COMM	= 0x85.2
#define IN		1
#define OUT		0

#define PULLUP	0x04		// Bit position of COMM pin (see WPU).

// Bit mask for Rx commands.
#define mRED	0x80
#define mGRN	0x40
#define mBLU	0x20
#define mWHT	0x10

#define NACK	0x00		// Negative response.

// Timer1 rates (4us per tick).
#define m2ms	500		// Would be nice to be able to define as (2ms / 4us).
#define m60ms	15000

// Define all the commands.
enum eCommands { 
  RESET			= 0,
  SET_PULLUP		= 1,
  SET_LED_ON_OFF	= 2,
  QUICK_SET_LED_LEVEL	= 3,
  SET_LED_RAMP_DECAY	= 4,
  SET_LED_LEVEL		= 5,
  START_TURBO_PING	= 6,
  RESET_PWM_TIMER	= 7,
  SET_ACK_RESPONSE	= 8,
  SET_2ND_ADDR		= 9,
  PULSE_LED		= 10
};


#define LEVELS	16
const byte bLevels[ LEVELS ] = {
  0, 	4, 	13, 	20, 
  28, 	35, 	48, 	59, 
  72, 	88, 	104, 	120, 
  143, 	167, 	199, 	255
};

// Function protos...
void init( void );              // Call to initialize everything.    
void vGetCharISR( void );       // Irq for new characters.	     
void vServiceLEDs( void );      // Keeps LEDs updated.		     
bool fChecksum( byte len );     // Compute checksum.
void vParsePacket( void );      // Parse the new packet.	     
void Tx( byte ch );             // Transmit a character onto the bus.

// Ramp the indexed LED.
void vRampLED(  byte *thersh, byte *tar, byte ramp, byte decay, byte iIndex );	




